from duplocloud.client import DuploClient
from duplocloud.resource import DuploResourceV3
from duplocloud.commander import Resource


@Resource("s3", scope="tenant")
class DuploS3(DuploResourceV3):
  def __init__(self, duplo: DuploClient):
    super().__init__(duplo, "aws/s3bucket")

  def name_from_body(self, body):
    return body["Name"]


import pytest
from unittest.mock import Mock
from duplocloud.commander import resources, Resource, Command, load_resource
from duplocloud.client import DuploClient
from duplocloud.resource import DuploResourceV2, DuploResourceV3

@pytest.mark.unit
def test_portal_scoped_resource():
  """Test that portal-scoped resources don't get tenant functionality injected."""
  
  @Resource("test_portal", "portal")
  class TestPortalResource:
    def __init__(self, duplo: DuploClient):
      self.duplo = duplo
    
    @Command()
    def list(self):
      pass
  
  # Check the resource is registered
  assert "test_portal" in resources
  assert resources["test_portal"]["scope"] == "portal"
  
  # Verify the class doesn't have tenant properties
  assert not hasattr(TestPortalResource, "tenant")
  assert not hasattr(TestPortalResource, "tenant_id")
  assert not hasattr(TestPortalResource, "tenant_svc")

@pytest.mark.unit
def test_tenant_scoped_resource():
  """Test that tenant-scoped resources get tenant functionality injected."""
  
  @Resource("test_tenant", "tenant")
  class TestTenantResource:
    def __init__(self, duplo: DuploClient):
      self.duplo = duplo
    
    @Command()
    def list(self):
      pass
  
  # Check the resource is registered with tenant scope
  assert "test_tenant" in resources
  assert resources["test_tenant"]["scope"] == "tenant"
  
  # Verify tenant functionality was injected
  assert hasattr(TestTenantResource, "tenant")
  assert hasattr(TestTenantResource, "tenant_id")
  assert hasattr(TestTenantResource, "prefixed_name")
  assert hasattr(TestTenantResource, "endpoint")
  
  # Verify these are properties
  assert isinstance(getattr(TestTenantResource, "tenant"), property)
  assert isinstance(getattr(TestTenantResource, "tenant_id"), property)

@pytest.mark.unit
def test_default_scope_is_portal():
  """Test that resources default to portal scope when not specified."""
  
  @Resource("test_default")
  class TestDefaultResource:
    def __init__(self, duplo: DuploClient):
      self.duplo = duplo
    
    @Command()
    def list(self):
      pass
  
  # Check the resource defaults to portal scope
  assert "test_default" in resources
  assert resources["test_default"]["scope"] == "portal"

@pytest.mark.unit
def test_service_resource_scope():
  """Test that the real service resource has tenant scope and proper inheritance."""
  
  # Load the actual service resource
  service_class = load_resource("service")
  
  # Check that service is registered with tenant scope
  assert "service" in resources
  resource_info = resources["service"]
  
  # Verify scope (this should be tenant after we update the decorator)
  # For now, it might not have scope, so we'll check what we can
  assert "class" in resource_info
  assert resource_info["class"] == "DuploService"
  
  # Check that the service class has tenant functionality
  assert hasattr(service_class, "__init__")
  
  # After scoping is implemented, these should exist:
  # assert hasattr(service_class, "tenant")
  # assert hasattr(service_class, "tenant_id")

@pytest.mark.unit
def test_tenant_mixin_properties_work():
  """Test that injected tenant properties actually work when instantiated."""
  
  @Resource("test_tenant_props", "tenant")
  class TestTenantPropsResource:
    def __init__(self, duplo: DuploClient):
      self.duplo = duplo
    
    @Command()
    def find(self, name: str):
      return {"name": name}
  
  # Create a mock client
  class MockClient:
    def __init__(self):
      self.tenant = "test-tenant"
      self.tenantid = "test-id"
    
    def load(self, name):
      class MockTenantService:
        def find(self):
          return {
            "TenantId": "mock-tenant-id",
            "AccountName": "mockaccount"
          }
      return MockTenantService()
  
  # Instantiate the resource
  resource = TestTenantPropsResource(MockClient())
  
  # Verify private attributes were initialized
  assert hasattr(resource, "_tenant")
  assert hasattr(resource, "_tenant_id")
  assert hasattr(resource, "tenant_svc")

@pytest.mark.unit
def test_parent_class_with_scope():
  """Test that parent class reference is maintained correctly."""
  
  class BaseResource:
    def __init__(self, duplo: DuploClient):
      self.duplo = duplo
    
    @Command()
    def base_method(self):
      pass
  
  @Resource("test_child", "tenant")
  class ChildResource(BaseResource):
    @Command()
    def child_method(self):
      pass
  
  # Check parent is recorded correctly
  assert "test_child" in resources
  assert resources["test_child"]["parent"] == "BaseResource"
  assert resources["test_child"]["scope"] == "tenant"
  
  # Verify tenant functionality is injected into child
  assert hasattr(ChildResource, "tenant")
  assert hasattr(ChildResource, "tenant_id")

@pytest.mark.unit
def test_invalid_scope_raises_error():
  """Test that invalid scope values raise an error."""
  
  with pytest.raises(ValueError) as exc_info:
    @Resource("test_invalid", "invalid_scope")
    class TestInvalidResource:
      pass
  
  assert "Invalid scope" in str(exc_info.value)

@pytest.mark.unit
def test_v2_portal_scope_endpoint():
  """Test that V2 portal-scoped resources use the base endpoint method."""
  mock_duplo = Mock()
  
  @Resource("testv2portal", scope="portal")
  class TestV2PortalResource(DuploResourceV2):
    pass
  
  # Directly instantiate instead of using load_resource
  instance = TestV2PortalResource(mock_duplo)
  
  # Test V2 portal endpoint format (just returns the path)
  assert instance.endpoint("some/path") == "some/path"

@pytest.mark.unit 
def test_v2_tenant_scope_endpoint():
  """Test that V2 tenant-scoped resources get the correct endpoint method."""
  mock_duplo = Mock()
  mock_duplo.tenantid = "test-tenant-456"
  
  @Resource("testv2tenant", scope="tenant")
  class TestV2TenantResource(DuploResourceV2):
    pass
  
  # Directly instantiate instead of using load_resource
  instance = TestV2TenantResource(mock_duplo)
  instance._tenant_id = "test-tenant-456"
  
  # Test V2 tenant endpoint format
  assert instance.endpoint("AdminProxy/GetJITAccess") == "subscriptions/test-tenant-456/AdminProxy/GetJITAccess"

@pytest.mark.unit
def test_v3_portal_scope_endpoint():
  """Test that V3 portal-scoped resources use the base endpoint method."""
  mock_duplo = Mock()
  
  @Resource("testv3portal", scope="portal")
  class TestV3PortalResource(DuploResourceV3):
    def __init__(self, duplo):
      super().__init__(duplo, slug="testresources", prefixed=False)
  
  # Directly instantiate instead of using load_resource
  instance = TestV3PortalResource(mock_duplo)
  
  # Test V3 portal endpoint format (no tenant_id)
  assert instance.endpoint() == "v3/testresources"
  assert instance.endpoint("myresource") == "v3/testresources/myresource"
  assert instance.endpoint("myresource", "subpath") == "v3/testresources/myresource/subpath"

@pytest.mark.unit
def test_v3_tenant_scope_endpoint():
  """Test that V3 tenant-scoped resources get the correct endpoint method."""
  mock_duplo = Mock()
  mock_duplo.tenantid = "test-tenant-123"
  
  @Resource("testv3tenant", scope="tenant")
  class TestV3TenantResource(DuploResourceV3):
    def __init__(self, duplo):
      super().__init__(duplo, slug="testresources", prefixed=False)
  
  # Directly instantiate instead of using load_resource
  instance = TestV3TenantResource(mock_duplo)
  instance._tenant_id = "test-tenant-123"
  
  # Test V3 tenant endpoint format
  assert instance.endpoint() == "v3/subscriptions/test-tenant-123/testresources"
  assert instance.endpoint("myresource") == "v3/subscriptions/test-tenant-123/testresources/myresource"
  assert instance.endpoint("myresource", "subpath") == "v3/subscriptions/test-tenant-123/testresources/myresource/subpath"



import pytest
from unittest.mock import patch
from duplo_resource.argo_wf import DuploArgoWorkflow


def _setup_argo_resource(mocker):
    """Helper to create ArgoWorkflow resource with mocked client."""
    mock_client = mocker.MagicMock()
    mock_client.host = "https://test.duplocloud.net"
    mock_client.token = "test-token"
    mock_client.timeout = 60
    mock_client.tenantid = None
    # Mock tenant service
    mock_tenant_svc = mocker.MagicMock()
    mock_tenant_svc.find.return_value = {"TenantId": "tenant-123", "AccountName": "test-tenant", "PlanID": "test-plan"}
    # Mock infrastructure service
    mock_infra_svc = mocker.MagicMock()
    mock_infra_svc.find.return_value = {
        "CustomData": [{"Key": "DuploCiTenant", "Value": "ci-tenant-id"}]
    }
    # Return appropriate service based on load call
    def load_service(name):
        if name == "tenant":
            return mock_tenant_svc
        elif name == "infrastructure":
            return mock_infra_svc
        return mocker.MagicMock()
    mock_client.load.side_effect = load_service
    # Mock auth response
    mock_auth = mocker.MagicMock()
    mock_auth.json.return_value = {"Token": "argo-token", "IsAdmin": True, "TenantId": "ctrl-tenant"}
    mock_client.post.return_value = mock_auth
    # Mock system info response
    mock_system = mocker.MagicMock()
    mock_system.json.return_value = {"ResourceNamePrefix": "msi"}
    mock_client.get.return_value = mock_system
    return DuploArgoWorkflow(mock_client), mock_client


@pytest.mark.unit
def test_auth(mocker):
    argo, mock_client = _setup_argo_resource(mocker)
    result = argo.auth()
    assert result["Token"] == "argo-token"
    mock_client.post.assert_called_once()


@pytest.mark.unit
def test_list_templates(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {"items": []})
        result = argo.list_templates()
        assert "items" in result


@pytest.mark.unit
def test_get_template(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {"metadata": {"name": "tpl1"}})
        result = argo.get_template("tpl1")
        assert result["metadata"]["name"] == "tpl1"


@pytest.mark.unit
def test_list_workflows(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {"items": []})
        result = argo.list_workflows()
        assert "items" in result


@pytest.mark.unit
def test_get_workflow(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {"metadata": {"name": "wf1"}})
        result = argo.get_workflow("wf1")
        assert result["metadata"]["name"] == "wf1"


@pytest.mark.unit
def test_submit(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {"metadata": {"name": "new-wf"}})
        result = argo.submit({"workflow": {}})
        assert result["metadata"]["name"] == "new-wf"


@pytest.mark.unit
def test_delete_workflow(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {})
        result = argo.delete_workflow("wf1")
        assert result == {}


@pytest.mark.unit
def test_create_template(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {"metadata": {"name": "tpl"}})
        result = argo.create_template({"template": {}})
        assert result["metadata"]["name"] == "tpl"


@pytest.mark.unit
def test_delete_template(mocker):
    argo, _ = _setup_argo_resource(mocker)
    with patch('requests.request') as mock_req:
        mock_req.return_value = mocker.MagicMock(status_code=200, json=lambda: {})
        result = argo.delete_template("tpl")
        assert result == {}



import requests
from duplocloud.client import DuploClient
from duplocloud.errors import DuploError
from duplocloud.resource import DuploResource
from duplocloud.commander import Command, Resource
import duplocloud.args as args

@Resource("argo_wf")
class DuploArgoWorkflow(DuploResource):
  """Resource for creating and managing Argo Workflow resources in DuploCloud.

  This resource provides commands to interact with Argo Workflows via the DuploCloud proxy.
  It handles the two-step authentication: first obtaining an Argo token from Duplo, then
  using that token to communicate with the Argo Workflow API.

  Usage: Basic CLI Use
    ```sh
    duploctl argo_wf <action>
    ```
  """

  def __init__(self, duplo: DuploClient):
    super().__init__(duplo)
    self._tenant = None
    self._tenant_id = None
    self._argo_auth = None
    self._system_info = None
    self._infra_config = None
    self.tenant_svc = duplo.load('tenant')
    self.infra_svc = duplo.load('infrastructure')

  @property
  def tenant(self):
    if not self._tenant:
      self._tenant = self.tenant_svc.find()
      self._tenant_id = self._tenant["TenantId"]
    return self._tenant

  @property
  def tenant_id(self):
    if not self._tenant_id:
      if self.duplo.tenantid:
        self._tenant_id = self.duplo.tenantid
      else:
        self._tenant_id = self.tenant["TenantId"]
    return self._tenant_id

  @property
  def namespace(self):
    prefix = self._get_resource_prefix()
    return f"{prefix}-{self.tenant['AccountName']}"

  def _ensure_argo_enabled(self):
    """Check if Argo Workflows feature is enabled for the tenant's infrastructure.

    Raises:
      DuploError: If Argo Workflows is not enabled (DuploCiTenant not configured).
    """
    if self._infra_config is None:
      plan_id = self.tenant.get("PlanID")
      if not plan_id:
        raise DuploError("Tenant has no associated infrastructure plan", 400)
      self._infra_config = self.infra_svc.find(plan_id)

    # DuploCiTenant is in CustomData array as {"Key": "DuploCiTenant", "Value": "..."}
    custom_data = self._infra_config.get("CustomData", [])
    ci_tenant = next(
      (item.get("Value") for item in custom_data if item.get("Key") == "DuploCiTenant"),
      None
    )
    if not ci_tenant:
      raise DuploError(
        "Argo Workflows is not enabled for this infrastructure. "
        "Please contact administrator.",
        400
      )

  def _get_resource_prefix(self) -> str:
    """Get the resource name prefix from system info.

    Returns:
      str: The resource name prefix (e.g., 'duploservices', 'msi')
    """
    if self._system_info is None:
      response = self.duplo.get("v3/features/system")
      self._system_info = response.json()
    return self._system_info.get("ResourceNamePrefix", "duploservices")

  def _get_argo_auth(self) -> dict:
    """Get Argo Workflow authentication token.

    Makes a POST call to the Duplo API to obtain an Argo Workflow token and admin status.
    The response includes a Kubernetes token for Argo API calls.

    Returns:
      dict: Authentication info containing Token, IsAdmin, TenantId, ExpiresAt

    Raises:
      DuploError: If Argo Workflows is not enabled for the infrastructure.
    """
    if self._argo_auth is None:
      self._ensure_argo_enabled()
      path = f"v3/auth/argo-wf/{self.tenant_id}/admin"
      response = self.duplo.post(path)
      self._argo_auth = response.json()
    return self._argo_auth

  def _argo_request(self, method: str, path: str, data: dict = None) -> dict:
    """Make a request to the Argo Workflow API.

    Uses the Argo token obtained from Duplo for authentication, and also
    passes the Duplo token in the 'duplotoken' header.

    Args:
      method: HTTP method (GET, POST, PUT, DELETE)
      path: API path (e.g., 'workflow-templates/namespace')
      data: Optional request body for POST/PUT

    Returns:
      dict: JSON response from the Argo API
    """
    auth = self._get_argo_auth()
    argo_tenant_id = auth["TenantId"]  # Tenant where workflow controller runs
    argo_token = auth["Token"]

    url = f"{self.duplo.host}/argo-wf/{argo_tenant_id}/api/v1/{path}?current_tenant_id={self.tenant_id}"
    headers = {
      'Content-Type': 'application/json',
      'Authorization': f'Bearer {argo_token}',
      'duplotoken': self.duplo.token
    }

    response = requests.request(
      method=method,
      url=url,
      headers=headers,
      json=data,
      timeout=self.duplo.timeout
    )

    if response.status_code == 404:
      raise DuploError("Argo resource not found", 404)
    if response.status_code == 401:
      raise DuploError("Unauthorized: Invalid or expired Argo token", 401)
    if response.status_code == 403:
      raise DuploError("Forbidden: Insufficient permissions for Argo operation", 403)
    if not (200 <= response.status_code < 300):
      raise DuploError(f"Argo API error: {response.text}", response.status_code)

    return response.json()

  @Command()
  def auth(self) -> dict:
    """Get Argo Authentication Info

    Retrieve the Argo Workflow authentication token and admin status for the current tenant.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf auth
      ```
    Returns:
      dict: Authentication info with Token, IsAdmin, TenantId, ExpiresAt
    """
    return self._get_argo_auth()

  @Command()
  def list_templates(self) -> list:
    """List Workflow Templates

    Retrieve a list of all workflow templates in the current tenant/namespace.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf list_templates
      ```

    Returns:
      list: A list of workflow templates.
    """
    path = f"workflow-templates/{self.namespace}"
    return self._argo_request("GET", path)

  @Command()
  def get_template(self, name: args.NAME) -> dict:
    """Get Workflow Template

    Retrieve a specific workflow template by name.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf get_template <name>
      ```

    Args:
      name: The name of the workflow template.

    Returns:
      dict: The workflow template object.
    """
    path = f"workflow-templates/{self.namespace}/{name}"
    return self._argo_request("GET", path)

  @Command()
  def list_workflows(self) -> list:
    """List Workflows

    Retrieve a list of all workflows in the current tenant/namespace.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf list_workflows
      ```

    Returns:
      list: A list of workflows.
    """
    path = f"workflows/{self.namespace}"
    return self._argo_request("GET", path)

  @Command()
  def get_workflow(self, name: args.NAME) -> dict:
    """Get Workflow

    Retrieve a specific workflow by name.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf get_workflow <name>
      ```

    Args:
      name: The name of the workflow.

    Returns:
      dict: The workflow object.
    """
    path = f"workflows/{self.namespace}/{name}"
    return self._argo_request("GET", path)

  @Command()
  def submit(self, body: args.BODY) -> dict:
    """Submit Workflow

    Submit a new workflow from a workflow spec.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf submit -f workflow.yaml
      ```
      Contents of the `workflow.yaml` file
      ```yaml
      --8<-- "src/tests/data/argo_workflow.yaml"
      ```

    Args:
      body: The workflow specification.

    Returns:
      dict: The created workflow object.
    """
    path = f"workflows/{self.namespace}"
    return self._argo_request("POST", path, body)

  @Command()
  def delete_workflow(self, name: args.NAME) -> dict:
    """Delete Workflow

    Delete a workflow by name.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf delete_workflow <name>
      ```

    Args:
      name: The name of the workflow to delete.

    Returns:
      dict: Deletion confirmation.
    """
    path = f"workflows/{self.namespace}/{name}"
    return self._argo_request("DELETE", path)

  @Command()
  def create_template(self, body: args.BODY) -> dict:
    """Create Workflow Template

    Create a new workflow template.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf create_template -f template.yaml
      ```
      Contents of the `template.yaml` file
      ```yaml
      --8<-- "src/tests/data/argo_wf_template.yaml"
      ```

    Args:
      body: The workflow template specification.

    Returns:
      dict: The created workflow template object.
    """
    path = f"workflow-templates/{self.namespace}"
    return self._argo_request("POST", path, body)

  @Command()
  def delete_template(self, name: args.NAME) -> dict:
    """Delete Workflow Template

    Delete a workflow template by name.

    Usage: Basic CLI Use
      ```bash
      duploctl argo_wf delete_template <name>
      ```

    Args:
      name: The name of the workflow template to delete.

    Returns:
      dict: Deletion confirmation.
    """
    path = f"workflow-templates/{self.namespace}/{name}"
    return self._argo_request("DELETE", path)